# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-01-04 21:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('afip', '0009_auto_20151214_1836'),
    ]

    operations = [
        migrations.AlterField(
            model_name='receiptpdf',
            name='pdf_file',
            field=models.FileField(blank=True, null=True, upload_to='receipts', verbose_name='pdf file'),
        ),
    ]
