# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-02-03 02:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('afip', '0010_receiptpdf_pdffile_uploadto'),
    ]

    operations = [
        migrations.AddField(
            model_name='receiptentry',
            name='vat',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='receipt_entries', to='afip.VatType', verbose_name='vat'),
        ),
    ]
