# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-06 07:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('afip', '0012_taxpayer_profile_one_to_one'),
    ]

    operations = [
        migrations.AddField(
            model_name='taxpayer',
            name='is_sandboxed',
            field=models.BooleanField(default=True, help_text='Indicates if this taxpayer interacts with the sandbox servers rather than the production servers', verbose_name='is sandboxed'),
            preserve_default=False,
        ),
    ]
